/***************************************************************************
  $RCSfile: ddvcard2.cpp,v $
  -------------------
  cvs         : $Id: ddvcard2.cpp,v 1.2 2004/09/09 23:36:50 aquamaniac Exp $
  begin       : Mon Apr 14 2003
  copyright   : (C) 2003 by Martin Preuss
  email       : martin@libchipcard.de

 ***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifdef __declspec
# if BUILDING_DLL
#  define DLLIMPORT __declspec (dllexport)
# else /* Not BUILDING_DLL */
#  define DLLIMPORT __declspec (dllimport)
# endif /* Not BUILDING_DLL */
#else
# define DLLIMPORT
#endif


#include <openhbci2/callback.h>
#include "ddvcard2.h"
#include "mediumddv2.h"
#include <string.h>

#include <gwenhywfar/debug.h>


namespace HBCI {



DDVCard2Plugin::DDVCard2Plugin(Hbci *api)
:MediumPlugin(api), _client(0){
  GWEN_WAITCALLBACK *cb;

  _client=LC_Client_new("DDVCard2Plugin", VERSION, 0);

  if (LC_Client_ReadConfigFile(_client, 0)) {
    DBG_ERROR(0, "Error reading configuration.\n");
    LC_Client_free(_client);
    _client=0;
    return;
  }

  // register callback
  cb=HBCI_Callback_new("", (HBCI_Hbci*)api);
  if (GWEN_WaitCallback_Register(cb)) {
    DBG_ERROR(0, "Could not register callback");
    return;
  }

}



DDVCard2Plugin::~DDVCard2Plugin(){
  LC_Client_free(_client);
}



Error DDVCard2Plugin::mediumCheck(string &name) const {
#if 0

#if (k_CHIPCARD_VERSION_MAJOR>0) || (k_CHIPCARD_VERSION_MINOR>8)
  Pointer<DDVCardTrader> trader;
  Pointer<DDVCard> hcard;
  CTCard *cardp;
  CTError err;
  Error herr;
  bool havecard;

  trader=new DDVCardTrader(hbci(),
			   false,
			   0,
			   0,
			   CHIPCARD_STATUS_INSERTED,
			   CHIPCARD_STATUS_INSERTED |
			   CHIPCARD_STATUS_LOCKED_BY_OTHER,
			   CHIPCARD_STATUS_INSERTED);
  err=trader.ref().start();
  if (!err.isOk()) {
    fprintf(stderr, "DDVCard2Plugin: Could not initialize trader");
    return Error("DDVCard2Plugin::mediumCheck",
		 ERROR_LEVEL_NORMAL,
		 HBCI_ERROR_CODE_MEDIUM,
		 ERROR_ADVISE_ABORT,
		 "could not start trader");
  }

  havecard=false;
  while (!havecard) {
    // try for 5 seconds to get a card
    err=trader.ref().getNext(cardp, 5);
    if (!err.isOk()) {
      if (err.code()==k_CTERROR_API &&
	  (err.subcode1()==CHIPCARD_ERROR_NO_TRANSPORT ||
	   err.subcode1()==CHIPCARD_ERROR_NO_REQUEST)) {
	fprintf(stderr,
		"DDVCard2Plugin: Service unreachable, maybe \"chipcardd\" is not running?\n");
	trader.ref().stop();
	return Error("DDVCard2Plugin::mediumCheck",
		     ERROR_LEVEL_NORMAL,
		     HBCI_ERROR_CODE_NO_CARD,
		     ERROR_ADVISE_ABORT,
		     "service unreachable");
      }

      if (!hbci()->interactor().ref().
          msgInsertMediumOrAbort((User*)0, MediumTypeCard)) {
	if (Hbci::debugLevel()>1)
	  fprintf(stderr,"DDVCard2Plugin: User aborted.\n");
	trader.ref().stop();
	return Error("DDVCard2Plugin::mediumCheck",
		     ERROR_LEVEL_NORMAL,
		     HBCI_ERROR_CODE_USER_ABORT,
		     ERROR_ADVISE_ABORT,
		     "No card, user aborted.");
      }
    } // if err is not ok
    else
      havecard=true;
  } // while !havecard

  trader.ref().stop();

  // ok, we have a card, now check it
  hcard=new DDVCard(*cardp, hbci());
  err=hcard.ref().openCard();
  if (!err.isOk()) {
    fprintf(stderr,"DDVCard2Plugin: Error opening card: %s\n",
	    err.errorString().c_str());
    return Error("DDVCard2Plugin::mediumCheck",
		 ERROR_LEVEL_NORMAL,
		 HBCI_ERROR_CODE_BAD_MEDIUM,
		 ERROR_ADVISE_ABORT,
		 "This card is not my type.");
  }
  if (name.empty()) {
    // no name given, so create one
    name=hcard.ref().cardNumber();
  }
  else {
    // name given, so compare against that stored on the card (serial number)
    if (name!=hcard.ref().cardNumber()) {
      hcard.ref().closeCard();
      return Error("DDVCard2Plugin::mediumCheck",
		   ERROR_LEVEL_NORMAL,
		   HBCI_ERROR_CODE_WRONG_MEDIUM,
		   ERROR_ADVISE_ABORT,
		   "Serial number does not match.");
    }
  }

  // close the card
  err=hcard.ref().closeCard();
  if (!err.isOk()) {
    fprintf(stderr,"DDVCard2Plugin: Error closing card: %s\n",
	    err.errorString().c_str());
    return Error("DDVCard2Plugin::mediumCheck",
		 ERROR_LEVEL_NORMAL,
		 HBCI_ERROR_CODE_MEDIUM,
		 ERROR_ADVISE_ABORT,
		 "Card error.");
  }

  return Error();
#else
  return Error("DDVCard2Plugin::mediumCheck",
	       ERROR_LEVEL_NORMAL,
	       HBCI_ERROR_CODE_UNSUPPORTED,
	       ERROR_ADVISE_ABORT,
	       "Need newer Libchipcard version for this to work");
#endif

#endif
  return Error();
}


Pointer<Medium> DDVCard2Plugin::mediumFactory(const string &name){
  Pointer<Medium> m;

  m=new MediumDDV2(this, hbci(), name);
  m.setDescription("DDVCard2Plugin::mediumFactory (MediumDDV2)");
  m.setObjectDescription("DDVCard2Plugin::mediumFactory (MediumDDV2)");
  return m;
}


Pointer<Medium> DDVCard2Plugin::createNewMedium(bool readonly,
					       int country,
					       const string &bankId,
					       const string &userid,
					       const string &name){
  Pointer<Medium> m;

  m=new MediumDDV2(this, hbci(), name);
  return m;
}


Pointer<Medium> DDVCard2Plugin::mediumFromConfig(GWEN_DB_NODE *cfg) {
  string typ;
  string name;
  Pointer<Medium> m;
  Error err;
  GWEN_DB_NODE *pgroup;

  typ=GWEN_DB_GetCharValue(cfg, "mediumtype",0, "");
  if (strcasecmp(typ.c_str(), pluginName().c_str())!=0) {
    throw Error("DDVCard2Plugin::mediumFromConfig",
		ERROR_LEVEL_NORMAL,
		HBCI_ERROR_CODE_INVALID,
		ERROR_ADVISE_DONTKNOW,
		"bad medium type");
  }
  name=GWEN_DB_GetCharValue(cfg,
                            "mediumname",
                            0,
                            GWEN_DB_GetCharValue(cfg, "cardnumber", 0, ""));
  m=new MediumDDV2(this, hbci(), name);

  // load properties
  pgroup=GWEN_DB_GetGroup(cfg,
                          GWEN_DB_FLAGS_DEFAULT |
                          GWEN_PATH_FLAGS_NAMEMUSTEXIST,
                          "properties");
  if (pgroup) {
    // set property "usekeypad"
    err=m.ref().setProperty("usekeypad",
                            GWEN_DB_GetCharValue(pgroup,
                                                 "usekeypad",0,
                                                 "yes"));
    if (!err.isOk())
      throw Error("DDVCard2Plugin::mediumFromConfig", err);
  } // if properties

  return m;
}


Error DDVCard2Plugin::mediumToConfig(Pointer<Medium> m,
				    GWEN_DB_NODE *cfg) {
  Pointer<MediumDDV2> mddv;
  string s;
  Error err;
  GWEN_DB_NODE *pgroup;

  try {
    mddv=m.cast<MediumDDV2>();
  }
  catch (Error xerr) {
    return Error("DDVCard2Plugin::mediumToConfig",xerr);
  }
  GWEN_DB_SetCharValue(cfg,
                       GWEN_DB_FLAGS_DEFAULT |
                       GWEN_DB_FLAGS_OVERWRITE_VARS,
                       "mediumtype",m.ref().mediumTypeName().c_str());
  GWEN_DB_SetCharValue(cfg,
                       GWEN_DB_FLAGS_DEFAULT |
                       GWEN_DB_FLAGS_OVERWRITE_VARS,
		  "mediumname",mddv.ref().mediumName().c_str());

  // save properties
  pgroup=GWEN_DB_GetGroup(cfg,
                          GWEN_DB_FLAGS_DEFAULT |
                          GWEN_DB_FLAGS_OVERWRITE_GROUPS,
                          "properties");
  if (!pgroup) {
    return HBCI::Error("DDVCard2Plugin::mediumToConfig",
		       ERROR_LEVEL_NORMAL,
		       HBCI_ERROR_CODE_MEDIUM,
		       ERROR_ADVISE_DONTKNOW,
		       "could not create properties group");
  }


  // save property "usekeypad"
  err=m.ref().getProperty("usekeypad", s);
  if (!err.isOk())
    return Error("DDVCard2Plugin::mediumToConfig", err);
  GWEN_DB_SetCharValue(pgroup,
                       GWEN_DB_FLAGS_DEFAULT|
                       GWEN_DB_FLAGS_OVERWRITE_VARS,
                       "usekeypad", s.c_str());

  return Error();
}


Error DDVCard2Plugin::checkVersion(Hbci *api) {
  int vmajor, vminor, vpl, vbuild;

  /* do some version checking
   * Please note that the constants used below are defined upon
   * compilation of this plugin. If you compile this plugin using
   * OpenHBCI 0.9.10 and try to use it on a system where OpenHBCI 0.9.9
   * is installed, this module will refuse to work. This plugin only
   * works with versions of OpenHBCI which are the same or newer than
   * the version used to compile this plugin.
   * If the major version differs then this module believes that the
   * version of OpenHBCI used is totally incompatible with the version
   * used to build this plugin.
   */
  api->libraryVersion(vmajor, vminor, vpl, vbuild);
  if (vmajor!=OPENHBCI_VERSION_MAJOR) {
    /* this is the major compatibility check. The major version is
     * incremented if a newer version is incompatible with older ones */
    fprintf(stderr,
	    " Different major versions, "
	    "please recompile DDVCard plugin.\n");
    return HBCI::Error("DDVCard Plugin",
		       ERROR_LEVEL_NORMAL,
		       HBCI_ERROR_CODE_MEDIUM,
		       ERROR_ADVISE_DONTKNOW,
		       "Major version does not match");
  }
  if (!(
	((vminor==OPENHBCI_VERSION_MINOR)&&(vpl>OPENHBCI_VERSION_PATCHLEVEL))
	||
	((vminor==OPENHBCI_VERSION_MINOR) &&
	 (vpl==OPENHBCI_VERSION_PATCHLEVEL) &&
	 (vbuild>=OPENHBCI_VERSION_BUILD))
       )
     ) {
    fprintf(stderr,
	    "This plugin needs OpenHBCI v"OPENHBCI_VERSION_FULL_STRING
	    " or newer.\n");
    return HBCI::Error("DDVCard Plugin",
		       ERROR_LEVEL_NORMAL,
		       HBCI_ERROR_CODE_MEDIUM,
		       ERROR_ADVISE_DONTKNOW,
		       "need OpenHBCI v"OPENHBCI_VERSION_FULL_STRING
		       " or newer");
  }

  return Error();
}



}; // namespace


extern "C" {

  int ddvcard2_openhbci_plugin_interface_version = OPENHBCI_PLUGIN_INTERFACE_VERSION;

  int ddvcard2_openhbci_module_current=DDVCARD2_MODULE_CURRENT;
  int ddvcard2_openhbci_module_revision=DDVCARD2_MODULE_REVISION;
  int ddvcard2_openhbci_module_age=DDVCARD2_MODULE_AGE;


  HBCI::Error ddvcard2_registerPlugin(HBCI::Hbci *hbci,
                                      HBCI::PluginManager *mgr,
                                      PluginType pt) {
    HBCI::Pointer<HBCI::Plugin> mp;
    HBCI::Error err;

    err=HBCI::DDVCard2Plugin::checkVersion(hbci);
    if (!err.isOk())
      return HBCI::Error("DDVCard Plugin",err);

    // version is ok, now register plugin
    mp=new HBCI::DDVCard2Plugin(hbci);
    return mgr->registerPlugin(mp);
  }

  /*
   * This function simply creates a medium plugin without registering it
   */
  HBCI::Pointer<HBCI::Plugin> ddvcard2_createPlugin(HBCI::Hbci *hbci,
                                                    HBCI::PluginManager *mgr){
    HBCI::Pointer<HBCI::Plugin> mp;
    HBCI::Error err;

    err=HBCI::DDVCard2Plugin::checkVersion(hbci);
    if (!err.isOk())
      throw HBCI::Error("DDVCard Plugin",err);

    mp=new HBCI::DDVCard2Plugin(hbci);
    mp.setDescription("DDVCard2Plugin");
    return mp;
  }


}; // extern "C"



